-- *********************************************************************************************************************
-- *                           (c) 2013 by White Elephant GmbH, Schaffhausen, Switzerland                              *
-- *                                               www.white-elephant.ch                                               *
-- *                                                                                                                   *
-- *    This program is free software; you can redistribute it and/or modify it under the terms of the GNU General     *
-- *    Public License as published by the Free Software Foundation; either version 2 of the License, or               *
-- *    (at your option) any later version.                                                                            *
-- *                                                                                                                   *
-- *    This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the     *
-- *    implied warranty of MERCHANTABILITY or FITNESS for A PARTICULAR PURPOSE. See the GNU General Public License    *
-- *    for more details.                                                                                              *
-- *                                                                                                                   *
-- *    You should have received a copy of the GNU General Public License along with this program; if not, write to    *
-- *    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.                *
-- *********************************************************************************************************************
-->Style: White_Elephant

with Interfaces.C;
with System;

package Win with Pure => True is

  type BYTE is new Interfaces.C.unsigned_char;

  subtype BOOL   is Interfaces.C.int;
  subtype CHAR   is Interfaces.C.char;
  subtype INT    is Interfaces.C.int;
  subtype LONG   is Interfaces.C.long;
  subtype WCHAR  is Interfaces.C.wchar_t;
  subtype UINT   is Interfaces.C.unsigned;
  subtype ULONG  is Interfaces.C.unsigned_long;
  subtype USHORT is Interfaces.C.unsigned_short;
  subtype WORD   is USHORT;
  subtype DWORD  is ULONG;

  subtype HANDLE    is System.Address;
  subtype HBITMAP   is HANDLE;
  subtype HICON     is HANDLE;
  subtype HINSTANCE is HANDLE;
  subtype HMENU     is HANDLE;
  subtype HWND      is HANDLE;
  subtype LPVOID    is HANDLE;
  subtype PTCHAR    is HANDLE;

  subtype LPCSTR  is System.Address;
  subtype LPCWSTR is System.Address;

  type CHAR_Array is array (Natural range <>) of aliased CHAR;

  type COLORREF is new Win.DWORD;

  type LONG_PTR  is range -(2 ** (Standard'address_size - 1)) .. (2 ** (Standard'address_size - 1) - 1);
  type ULONG_PTR is mod 2 ** Standard'address_size;
  type UINT_PTR  is new ULONG_PTR;

  subtype WPARAM  is UINT_PTR;
  subtype LPARAM  is LONG_PTR;
  subtype LRESULT is LONG_PTR;

  Alignment : constant := Standard'address_size / System.Storage_Unit;

  FALSE : constant BOOL := 0;
  TRUE  : constant BOOL := 1;

  OK : constant LRESULT := 1;

  WM_USER    : constant := 16#400#;
  WM_VSCROLL : constant := 16#115#;

  CFM_COLOR : constant := 16#40000000#;

  EM_EXSETSEL      : constant := WM_USER + 55;
  EM_REPLACESEL    : constant := 16#c2#;
  EM_SETCHARFORMAT : constant := WM_USER + 68;

  ES_MULTILINE : constant := 16#4#;
  ES_READONLY  : constant := 16#800#;
  ES_SAVESEL   : constant := 16#00008000#;
  ES_SUNKEN    : constant := 16#00004000#;

  LF_FACESIZE : constant := 32;

  SB_BOTTOM : constant := 7;

  SCF_SELECTION : constant := 16#0001#;

  WS_CHILD   : constant := 16#40000000#;
  WS_HSCROLL : constant := 16#100000#;
  WS_VSCROLL : constant := 16#200000#;

  Charformat_Size : constant := 60;
  type CHARFORMAT is record
    Size           : UINT  := Charformat_Size;
    Mask           : DWORD;
    Effects        : DWORD;
    Height         : LONG;
    Offset         : LONG;
    Textcolor      : COLORREF;
    Charset        : BYTE;
    Pitchandfamily : BYTE;
    Facename       : CHAR_Array (0..LF_FACESIZE -1);
    Padding        : WORD;
  end record
  with
    Alignment  => Alignment,
    Convention => C,
    Pack       => Standard.True,
    Size       => Charformat_Size * BYTE'size;

  type CHARRANGE is record
    Min : LONG;
    Max : LONG;
  end record
  with
    Alignment  => Alignment,
    Convention => C,
    Pack       => Standard.True;

  type RECT is record
    Left   : LONG;
    Top    : LONG;
    Right  : LONG;
    Bottom : LONG;
  end record
  with
    Alignment  => Alignment,
    Convention => C,
    Pack       => Standard.True;


  function RGB (Red, Green, Blue : BYTE) return COLORREF;


  function Send_Message (Handle : HWND;
                         Msg    : UINT;
                         Wpar   : WPARAM;
                         Lpar   : LPARAM) return LRESULT;

  procedure Send_Message (Handle : HWND;
                          Msg    : UINT;
                          Wpar   : WPARAM;
                          Lpar   : LPARAM);

  function Load_Bitmap (Instance    : HINSTANCE;
                        Bitmap_Name : String) return HBITMAP;

  function Load_Library (Lib_File_Name : Wide_String) return HINSTANCE;

  function Create_Window (Class_Name  : Wide_String;
                          Window_Name : Wide_String;
                          Style       : DWORD;
                          X           : INT;
                          Y           : INT;
                          Width       : INT;
                          Height      : INT;
                          Wnd_Parent  : HWND;
                          Menu        : HMENU;
                          Instance    : HINSTANCE;
                          Param       : LPVOID) return HWND;
end Win;
